
##0.0 安装需要的R包
#（第一次使用时才需要运行此部分代码。安装成功后不需要重复安装）
install.packages('car')
install.packages('rms')
install.packages('pROC')
##0.1 调用R包
#清除R中的所有数据（归零）
rm(list = ls())

library(car)
library(rms)
library(pROC)

##0.2 设置数据存放的文件夹目录
#注意文件夹路径的斜杠是双斜杠\\
data_dir <- choose.dir(default = "D:\\data_dir", caption = "选择数据存放的文件夹目录")


##0.3 设置导出图片的文件夹目录
#注意文件夹路径的斜杠是双斜杠\\
output_dir <- choose.dir(default = "D:\\output_dir", caption = "选择导出图片的文件夹目录")


##0.4 选择数据文件（模板为logistic_training_dataset.txt）
#数据文件格式要求：注意数据文件要保存为txt版本
#数据格式要求：1. 数据中不要有中文字符；
#              2. 数据中第一行为变量名；
#              3. 数据中第一列为预测结局二分类指标（结局指标必须设置为0/1形式）；
#              4. 数据中从第二列开始为其他变量；
#              5. 请保持训练集和验证集中相同变量的名称一致
#              6. 空值以NA填充；二分类变量以0/1表示；多分类变量进行哑变量处理后以0/1表示；
#模板为生成的仿真数据

#选择文件路径
training_dataset_path <- choose.files(default = data_dir, caption = "请选训练集数据文件的txt文件。",
                                      multi = TRUE, filters = Filters,
                                      index = nrow(Filters))

#读取数据
training_dataset<- read.csv(training_dataset_path, header = TRUE,sep="\t", stringsAsFactors=FALSE)

#查看数据
print(paste0("该训练集有 ",dim(training_dataset)[1]," 个样本； ",dim(training_dataset)[2]," 个变量"))

################  设置ROC图中每条曲线的参数 ##########################
colnames(training_dataset)

#设置ROC图中每条曲线的参数   注意：更换数据集即可!!!!!!
function_list<-c(
  "Outcome ~ Diameter_2vs1+Diameter_3vs1+CT.value_2vs1+CT.value_3vs1+CT.value_4vs1+CTR_J+CEA_J"
)

#设置结局事件的变量名
outcome_name<-"Outcome"
#设置每条ROC的名称
ROC_names_list<-c("Training Cohort")#"1","2")
#设置每条ROC的颜色（选颜色可参考https://www.peisebiao.cn/）
ROC_color_list<-c("#00B39E")#,"#009EFA","#F47142")

#下面中括号中的内容直接运行
{
  ddist <- datadist(training_dataset)
  options(datadist='ddist')
  #构建多因素模型
  lrm_list<-list()
  for(index in 1:length(function_list)){
    lrm_list[[index]]<-lrm(as.formula(function_list[index]), data=training_dataset, x=TRUE, y=TRUE,maxit=5000)
  }
  #计算预测值及ROC曲线数值
  pred_f_training<-list()
  modelroc<-list()
  for(index in 1:length(function_list)){
    pred_f_training[[index]]<-predict(lrm_list[[index]],training_dataset)
    modelroc[[index]] <- roc(training_dataset[,outcome_name],pred_f_training[[index]])
  }
  #计算AUC并添加到名称中
  ROC_names_list_training<-ROC_names_list
  for(index in 1:length(function_list)){
    ROC_names_list_training[index]<-paste0(ROC_names_list_training[index]," (AUC=",round(modelroc[[index]]$auc[[1]],3),")")
  }
  names(modelroc)<-ROC_names_list_training
  #绘制多个ROC函数
  roc_list<-ggroc(modelroc, legacy.axes=TRUE,size = 1)+ #此处size改线条粗细
    scale_colour_manual(values = ROC_color_list)+
    annotate(geom = "segment",x=0,y=0,xend = 1,yend = 1)+ 
    theme_bw()
}

#导出ROC到设置的output_dir目录下
pdf(file=paste(output_dir, "\\ROC_multiplot_training.pdf", sep = ""),width=12,height=10) 
roc_list
dev.off()

