##数据处理
rm(list = ls())
library(rms)
library(readr)
data <- read_csv("D:/R work/Training-1.csv")
data_in <- read_csv("D:/R work/Internal validation-1.csv")
data_out <- read_csv("D:/R work/external validation-1.csv")

data$Outcome<-factor(data$Outcome,levels = c(0,1),labels = c("风险程度低","风险程度高"))
data$CTR<-factor(data$CTR,levels = c(0,1),labels = c("≤0.265",">0.265"))
data$Nodule_diameter<-factor(data$Nodule_diameter,levels = c(1,2,3),labels = c("≤10mm","10-20mm","20-30mm"))
data$CT_value<-factor(data$CT_value,levels = c(1,2,3,4),labels = c("≤-600HU","-600~-400HU","-400HU~-200HU",">-200HU"))
data$CEA<-factor(data$CEA,levels = c(0,1),labels = c("≤1.965ng/ml",">1.965ng/ml"))

data_in$Outcome<-factor(data_in$Outcome,levels = c(0,1),labels = c("风险程度低","风险程度高"))
data_in$CTR<-factor(data_in$CTR,levels = c(0,1),labels = c("≤0.265",">0.265"))
data_in$Nodule_diameter<-factor(data_in$Nodule_diameter,levels = c(1,2,3),labels = c("≤10mm","10-20mm","20-30mm"))
data_in$CT_value<-factor(data_in$CT_value,levels = c(1,2,3,4),labels = c("≤-600HU","-600~-400HU","-400HU~-200HU",">-200HU"))
data_in$CEA<-factor(data_in$CEA,levels = c(0,1),labels = c("≤1.965ng/ml",">1.965ng/ml"))

data_out$Outcome<-factor(data_out$Outcome,levels = c(0,1),labels = c("风险程度低","风险程度高"))
data_out$CTR<-factor(data_out$CTR,levels = c(0,1),labels = c("≤0.265",">0.265"))
data_out$Nodule_diameter<-factor(data_out$Nodule_diameter,levels = c(1,2,3),labels = c("≤10mm","10-20mm","20-30mm"))
data_out$CT_value<-factor(data_out$CT_value,levels = c(1,2,3,4),labels = c("≤-600HU","-600~-400HU","-400HU~-200HU",">-200HU"))
data_out$CEA<-factor(data_out$CEA,levels = c(0,1),labels = c("≤1.965ng/ml",">1.965ng/ml"))

View(data)
summary(data)
str(data)

#模型
formula1<-as.formula(Outcome~Nodule_diameter+CT_value+CTR+CEA)

#数据打包
dd=datadist(data)
options(datadist="dd")

dd_in=datadist(data_in)
options(datadist="dd_in")

dd_out=datadist(data_out)
options(datadist="dd_out")


#模型拟合
fit1<-lrm(formula1,data=data,x=T,y=T)
fit2<-lrm(formula1,data=data_in,x=T,y=T)
fit3<-lrm(formula1,data=data_out,x=T,y=T)
fit1
fit2
fit3


#AUC计算
data$predvalue1<-predict(fit1)
library(pROC)
ROC1 <- roc(data$Outcome,data$predvalue1)
round(auc(ROC1),3)
round(ci(auc(ROC1)),3)

data_in$predvalue2<-predict(fit2)
library(pROC)
ROC2 <- roc(data_in$Outcome,data_in$predvalue2)
round(auc(ROC2),3)
round(ci(auc(ROC2)),3)

data_out$predvalue3<-predict(fit3)
library(pROC)
ROC3 <- roc(data_out$Outcome,data_out$predvalue3)
round(auc(ROC3),3)
round(ci(auc(ROC3)),3)

#ROC绘制
plot(1-ROC1$specificities,ROC1$sensitivities,type = "l",col="red",lty=1,xlab = "FP",ylab = "TP",lwd=2)
abline(0,1)
legend(0.55,0.35,
       c("Model"),
       lty = c(1),
       lwd = c(2),
       col = c("red"),
       bty = "n")   ##o为加边框
###多条曲线方法1

data$predvalue1<-predict(fit1)
data$predvalue2<-predict(fit2)
ROC1 <- roc(data$Outcome,data$predvalue1)
ROC2 <- roc(data$Outcome,data$predvalue2)

plot(1-ROC1$specificities,ROC1$sensitivities,type = "l",col="red",lty=1,xlab = "FP",ylab = "TP",lwd=2)
lines(1-ROC2$specificities,ROC2$sensitivities,col="blue",lty=1,lwd=2)
lines(1-ROC3$specificities,ROC3$sensitivities,col="yellow",lty=1,lwd=2)
abline(0,1)
legend(0.6,0.6,
       c("Trainging","Internal validation","External validation"),
       lty = c(1,1,1),
       lwd = c(2,2,2),
       col = c("red","blue","yellow"),
       bty = "n") 
##查看AUC和CI
round(ci(auc(ROC1)),3)
round(ci(auc(ROC2)),3)
round(ci(auc(ROC3)),3)

###多条曲线方法2
library(riskRegression)
fit1=glm(formula1,data=data,family = binomial())
fit2=glm(formula1,data=data_in,family = binomial())
fit3=glm(formula1,data=data_out,family = binomial())

xb <-Score(list("Trainging"=fit1,
                "Internal validation"=fit2,
                "External validation"=fit3),
           formula=Outcome~1,
           null.model=FALSE,
           conf.int=TRUE,
           plots=c("calibration","ROC"),
           metrics=c("auc","brier"),
           B=1000,M=50,
           data=data)
plotROC(xb)
plotCalibration(xb)